//@ts-check
"use strict";

/**
 * @typedef ItemDesc
 * @type {{
 *   id: ItemName;
 * }}
 */

/**
 * @typedef ItemName
 * @type {keyof typeof itemDescs}
 */

/**
@type {{
  bow: ItemDesc,
  sword: ItemDesc,
  mace: ItemDesc,
  dagger: ItemDesc,
  ancientBread: ItemDesc,
  bread: ItemDesc,
  cheese: ItemDesc,
  ham: ItemDesc,
  board: ItemDesc,
  boardBag: ItemDesc,
  money: ItemDesc,
  dragonEgg: ItemDesc,
  torch: ItemDesc,
}}
 */
const itemDescs = {
  "bow": {
    id: "bow",
  },
  "sword": {
    id: "sword",
  },
  "mace": {
    id: "mace",
  },
  "dagger": {
    id: "dagger",
  },
  "ancientBread": {
    id: "ancientBread",
  },
  "bread": {
    id: "bread",
  },
  "cheese": {
    id: "cheese",
  },
  "ham": {
    id: "ham",
  },
  "board": {
    id: "board",
  },
  "boardBag": {
    id: "boardBag",
  },
  "money": {
    id: "money",
  },
  "dragonEgg": {
    id: "dragonEgg",
  },
  "torch": {
    id: "torch",
  },
}

const itemMetrics = {
  height: 24,
};

/**
@typedef Item
@type {{
  id: string,
  desc: ItemDesc,
  pos: PIXI.Point,
  pixiRoot: PIXI.Container,
  sprite: PIXI.Sprite,
  outlineFilter: PIXI.filters.OutlineFilter,
}}
 */

/**
 * @param {World} world
 * @param {PointLike} pos
 * @param {ItemDesc} desc
 * @returns Item
 */
function createItem(world, pos, desc) {
  const pixiRoot = new PIXI.Container();
  pixiRoot.scale.set(inverseTileSize);
  pixiRoot.position.set(pos.x, pos.y);
  world.pixiLayers["ground"].entitiesContainer.addChild(pixiRoot);

  const sprite = new PIXI.Sprite(tex(`inGame/items/${desc.id}`));
  let outlineFilter = new PIXI.filters.OutlineFilter(0, 0xffffff, 0.5);
  sprite.filters = [outlineFilter];
  sprite.anchor.set(0.5, 1);
  pixiRoot.addChild(sprite);

  /** @type {Item} */
  const item = {
    id: generateID(),
    desc,
    pos: new PIXI.Point(pos.x, pos.y),
    pixiRoot,
    sprite,
    outlineFilter,
  };

  return item;
}

/** @param {number} dt */
function updateItems(dt) {
  const { items } = inGame.activeWorld;
  const { hovered } = inGame.stats;

  for (const item of items) {
    let isHovered = (hovered && hovered.kind == "item" && hovered.id == item.id);
    item.outlineFilter.thickness = (isHovered ? 1 : 0) * game.stageScale;
  }
}

/**
 * @param {Item} item
 */
function destroyItem(item) {
  const world = inGame.activeWorld;
  world.pixiLayers["ground"].entitiesContainer.removeChild(item.pixiRoot);
  world.items = world.items.filter((x) => x !== item);
}

/** @returns {number} */
function countEmptyItemSlots() {
  return inGame.stats.itemSlots.filter((slot) => !slot.item).length;
}

/**
 * @param {ItemName} itemName
 * @returns {boolean}
 */
function pickupItem(itemName) {
  const { stats } = inGame;
  let slotNumber = stats.itemSlots.findIndex((slot) => !slot.item);
  if (slotNumber === -1) {
    notify("error", "No room in inventory!");
    getSound("baxton.inventoryFull").play();
    return false;
  }

  stats.itemSlots[slotNumber].item = itemName;
  updateHudItem(slotNumber);

  getSound("pickUp.item").play();
  return true;
}

/**
 * @param {ItemName} itemName
 */
function dropItem(itemName) {
  const { stats } = inGame;
  let slotNumber = stats.itemSlots.findIndex((slot) => slot && slot.item === itemName);
  if (slotNumber !== -1) {
    stats.itemSlots[slotNumber].item = null;
    updateHudItem(slotNumber);
  }
}
